----------------------------------------------------------------------
-- Fmdac.vhd
-- Description:   
-- Author:        Carsten Wulff <wulff@iet.ntnu.no>
-- Created at:    Fri Sep 15 08:47:17 2006
-- Modified at:   Thu Jan 31 11:20:25 2008
-- Modified by:   Carsten Wulff <wulff@iet.ntnu.no>
----------------------------------------------------------------------
-- Mulitplying digital to analog converter
-- This is a amplifier that performs the function
-- Vout = Vin*2 - Vref, Vin >= 1/4Vref
-- Vout = Vin*2 , Vin < 1/4Vref & Vin >= -1/4Vref
-- Vout = Vin*2 + Vref, Vin > -1/4Vref
-- It usually entails a switched capacitor circuit 
-- Take a look at http://ieeexplore.ieee.org/search/srchabstract.jsp?arnumber=1458994&isnumber=31404&punumber=4&k2dockey=1458994@ieeejrns
-- for some nice schematics.
library IEEE;
use IEEE.math_real.all;
use IEEE.electrical_systems.all;
USE IEEE.STD_LOGIC_1164.ALL;
entity MDAC is
  port (
   terminal vin, vip,vcm,vrn,vrp,outn,outp : Electrical;     --Analog Inputs
   signal phi1,phi1a,phi2 : in std_logic;     -- Clock Signals
   signal t : in std_logic_vector(0 to 2)      -- From SADC
   );
end MDAC;

architecture ideal of MDAC is
  terminal vref : electrical;
  -- Define voltages and currents with quantities
  terminal vrefn : electrical;
  quantity q_vin across vin;
  quantity q_vip across vip;
  quantity q_vcm across vcm;
  quantity q_vrn across vrn;
  quantity q_vrp across vrp;
  quantity q_outn across i_outn through outn;
  quantity q_outp across i_outp through outp;
  quantity q_vref across i_vref through vref;
  -- Internal analog signals
  signal   s_outn, s_outp : real := 0.5;
  signal   s_vin, s_vip     : real := 0.0;
  signal   s_refn, s_refp   : real := 0.0;
begin
  --Set the output quantities to the output signals
  q_outn  == s_outn'ramp(10.0e-12);
  q_outp  == s_outp'ramp(10.0e-12);
  --Define the reference quantity
  q_vref == (q_vrp - q_vrn)/2.0;

  -- Sample input, remove the common mode
  sample : process (phi1a)
  begin 
    if phi1a'event and phi1a = '0' then
      s_vin <= q_vin - q_vcm;
      s_vip <= q_vip - q_vcm;
    end if;
  end process sample;

-------------------------------------------------------------------------------
--  The MDAC implements the function shown below
-------------------------------------------------------------------------------
--  Vref   |                          /
--         |                        /
--  Vref/2 |          /|    /|    /
--         |        /  |  /  |  /
-- -Vref/2 |      /    |/    |/
--         |    /
-- -Vref   |  /
--         -----------------------------
--          -Vref  -Vref/4  Vref/4  Vref
--
-- Mulitply the input and subtract/add/do-nothing the reference voltage
  multiply : process (phi2)
variable gain : real := 2.0;
  begin
    if phi2'event and phi2 = '1' then  
      case t is
        when "100" =>
          s_outp <= q_vcm + s_vip*gain - q_vref;
          s_outn <= q_vcm + s_vin*gain + q_vref;
        when "001" =>
          s_outp <= q_vcm + s_vip*gain + q_vref;
          s_outn <= q_vcm + s_vin*gain - q_vref;
        when others =>
          s_outp <= q_vcm + s_vip*gain;
          s_outn <= q_vcm + s_vin*gain;
      end case;      
    end if;
  end process multiply;
end ideal;
